/*! \brief C++ implementation of EDFB
 *
 *  This code aims at replicating the original work-flow in C++.
 */

#include <cstdio>
#include <cmath>
#include <complex>
#include <cstring>
#include <iostream>
#include <fstream>
#include "List.h"

using namespace std;

string *load_file(string file_name, int *count);

int main(int argc, char **argv) {
  // Common variables set
  complex<double> *dc0, *dc0m;
  double *ros, **rcf;
  int *iog, *nshl;
  double *xiv, *wns, *wls, *pus, *evs, *vss;
  string vns[5];

  // Input file reading section
  int num_lines = 0;
  int last_read_line; //!< Keep track of where INXI left the input stream
  string *file_lines = load_file("../../test_data/sphere/DEDFB", &num_lines);

  // Configuration code
  int nsph, ies;
  sscanf(file_lines[0].c_str(), " %d %d", &nsph, &ies);
  if (ies != 0) ies = 1;
  double exdc, wp, xip;
  int exdc_exp, wp_exp, xip_exp;
  int idfc, nxi, instpc, insn;
  sscanf(
    file_lines[1].c_str(),
    " %9lf D%d %9lf D%d %8lf D%d %d %d %d %d",
    &exdc, &exdc_exp,
    &wp, &wp_exp,
    &xip, &xip_exp,
    &idfc, &nxi, &instpc, &insn
  );
  exdc *= pow(10.0, exdc_exp);
  wp *= pow(10.0, wp_exp);
  xip *= pow(10.0, xip_exp);

  FILE *output = fopen("c_OEDFB", "w");
  // FORTRAN starts subroutine INXI at this point
  const double pigt = acos(0.0) * 4.0;
  const double evc = 6.5821188e-16;
  if (idfc >= 0) {
    printf("Not walked by input data.\n");
  } else {
    if (instpc < 1) {
      // In this case the XI vector is explicitly defined.
      // Test input comes this way.
      vns[insn] = "XIV";
      List<double> xi_vector;
      double xi;
      int xi_exp;
      sscanf(file_lines[2].c_str(), " %9lE D%d", &xi, &xi_exp);
      xi *= pow(10.0, xi_exp);
      xi_vector.set(0, xi);
      for (int jxi310 = 1; jxi310 < nxi; jxi310++) {
	sscanf(file_lines[2 + jxi310].c_str(), " %9lE D%d", &xi, &xi_exp);
	xi *= pow(10.0, xi_exp);
	xi_vector.append(xi);
	last_read_line = 2 + jxi310;
      }
      vss = xi_vector.to_array();
      xiv = xi_vector.to_array();
      double pu = xip + wp;
      double wn = pu / 3.0e8;
      fprintf(output, "          XIP          WN           WL           PU           EV\n");
      fprintf(output, "     %13.4lE", xip);
      fprintf(output, "%13.4lE", wn);
      fprintf(output, "%13.4lE", pigt / wn);
      fprintf(output, "%13.4lE", pu);
      fprintf(output, "%13.4lE\n", pu * evc);
      fprintf(output, "  SCALE FACTORS XI\n", pu * evc);
      for (int jxi6612 = 1; jxi6612 <= nxi; jxi6612++)
	fprintf(output, "%5d%13.4lE\n", jxi6612, xiv[jxi6612 - 1]);
      //INXI branch ends here.
    }
  }
  last_read_line++;
  iog = new int[nsph];
  for (int i = 0; i < nsph; i++) {
    sscanf(file_lines[last_read_line].c_str(), " %d", (iog + i));
  }
  nshl = new int[nsph];
  ros = new double[nsph];
  rcf = new double*[nsph];
  for (int i113 = 1; i113 <= nsph; i113++) {
    int i_val;
    double ros_val;
    int ros_val_exp;
    if (iog[i113 - 1] < i113) continue;
    sscanf(file_lines[++last_read_line].c_str(), " %d %9lf D%d", &i_val, &ros_val, &ros_val_exp);
    nshl[i113 - 1] = i_val;
    ros[i113 - 1] = ros_val * pow(10.0, ros_val_exp);
    int nsh = nshl[i113 -1];
    if (i113 == 1) nsh += ies;
    rcf[i113 - 1] = new double[nsh];
    for (int ns = 0; ns < nsh; ns++) {
      double ns_rcf;
      int ns_rcf_exp;
      sscanf(file_lines[++last_read_line].c_str(), " %8lf D%d", &ns_rcf, &ns_rcf_exp);
      rcf[i113 -1][ns] = ns_rcf * pow(10.0, ns_rcf_exp);
    }
  }
  if (idfc < 0) {
    fprintf(output, "  DIELECTRIC CONSTANTS\n");
  }
  fclose(output);
  return 0;
}

string *load_file(string file_name, int *count) {
  fstream input_file(file_name.c_str(), ios::in);
  List<string> file_lines = List<string>();
  string line;
  if (input_file.is_open()) {
    getline(input_file, line);
    file_lines.set(0, line);
    while (getline(input_file, line)) {
      file_lines.append(line);
    }
    input_file.close();
  }
  string *array_lines = file_lines.to_array();
  *count = file_lines.length();
  return array_lines;
}
